import java.io.*;
import java.awt.*;

import java.awt.font.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.Timer;

import java.util.Scanner;

/*
 * COSC102   Lab 8
 *
 * Speed Reader 
 *
 * The starting code does the following two things
 *    1. After 5 sec there is a blinking square, which color
 *       alternates between black and red every 1 sec. This animation is achieved
 *       thanks to a Timer object connected to the SpeedReader object, this, as 
 *       it implements the actionPerformed method of the ActionListener interface.
 *    
 *    2. Pressing the spacebar key changes the word displayed from the instance
 *        variable array, fixStrings.
 * 
 * Your task is to implement a speed reader application
 *
 * 1. Implement the command line arguments. See comments: COMMAND LINE ARGS 
 * 2. Process the words of the file. See comments: WORDS
 * 3. Display them using the font size and at the speed requested: each word 
 *    is displayed at the center of the frame and with the middle letter colored. 
 *    See comments: DISPLAY
 * 
 *
 * Credit: Nifty Assignments written by Peter-Michael Osera
 * http://nifty.stanford.edu/2015/posera-speed-reader/
 *
 * Provided code
 * @author Elodie Fourquet 
 * @date Mars, 2015
 * */

public class SpeedReader extends JComponent implements ActionListener {
 
  public static final int FRAME_WIDTH = 800;
  public static final int FRAME_HEIGHT = 200;
  public static final int REC_SIZE = 40;
   
  public static final int SEC_PER_MIN = 60;
  public static final int MILLIS_SEC_PER_MIN = SEC_PER_MIN * 1000;
  
  public static final int INIT_DELAY = 5000;
  
  //WORDS: you have to replace this hard-coded arrays
  //with the tokens you get from the file provided on the command line
  //Think about an instance variable and method(s) (maybe helper one)
  private String[] fixStrings = {"to", "or", "not", "to", "be", 
                            "that", "is", "the", "question"};
  private int wordIndex = 0;
  private Font font;   //A Monospace font has to be used
  
  private Color smallRectColors[] = {
    Color.WHITE, Color.RED
  };  
  private int colorIndex = 0;
  
  public SpeedReader(Scanner scan, int fontSize, int wpm) {
    
    super();

    int inBwWordDelay = delay(wpm); //1.b DISPLAY: does it make sense?
    System.out.println("wpm " + wpm + " ms delay " + inBwWordDelay); 
    
    initListener();
    
    //WORDS: do something with scan; add a method
    
    //  http://docs.oracle.com/javase/7/docs/api/javax/swing/Timer.html
    Timer timer = new Timer(INIT_DELAY, this);   //init delay of 5 sec
    timer.setDelay(inBwWordDelay);         //1.c DISPLAY: tweak those values
    timer.start();
    
    // Needs to be a Monospace font
    font = new Font("Courier", Font.BOLD, 42); //DISPLAY: what is 
                                            // this hard-coded value 42?
                                            //check the Font API
    
  }
  
  //returns the millisecond value corresponding to words/minute
  //1.a DISPLAY: implement it
  private static int delay(int wpm) {
    return 1000;    //1.a DISPLAY: do the conversion, 
                    //arbitrary value returned for now
  }
  
  
  private void initListener() {
    setFocusable(true);
    requestFocus();

    addKeyListener(new KeyAdapter() {
  
      public void keyPressed(KeyEvent ae) {
        //System.out.println("key " + ae.getKeyChar() 
                                    // + " " + ae.getKeyCode());
       
        if (ae.getKeyCode()==KeyEvent.VK_Q)
          System.exit(1);
        else if (ae.getKeyCode()==KeyEvent.VK_SPACE) {
          wordIndex = (wordIndex+1) % fixStrings.length;
          repaint();
        }
      }
    });
  }
  
  /**
   * Displays our component with the animated square 
   */
  @Override
  protected void paintComponent(Graphics g) {
    super.paintComponent(g);
    
    Graphics2D g2d = (Graphics2D)g;
    
    g2d.setColor(Color.YELLOW);    //repaint background to erase each time
    g2d.fillRect(0, 0, getWidth(), getHeight()); 
    
    int smallRectX =  2 * getWidth()/3;
    int smallRectY = getHeight() / 2;
    g2d.setColor(smallRectColors[colorIndex]);
    g2d.fillRect(smallRectX, smallRectY, REC_SIZE, REC_SIZE);
 
    g2d.setFont(font);
    
    String word = fixStrings[wordIndex];
    int oriX = 50;
    int oriY = 100;
    //DISPLAY: understand the following to be able to update the position 
    //and color according to the center character
    //The class FontMetrics is useful
    //http://docs.oracle.com/javase/7/docs/api/java/awt/FontMetrics.html

    g2d.setColor(Color.BLACK);                   
    String subString = word.substring(0, 1);
    g2d.drawString(subString, oriX, oriY);

    FontMetrics metrics = g2d.getFontMetrics();
    int width = metrics.stringWidth(subString);
    g2d.setColor(Color.RED);
    g2d.drawString(word.substring(1), oriX + width, oriY);

    g2d.dispose();
  }
  
  /**
   * Handles Timer events by toggling colorIndex 
   * which is used to fill the rectangle
   */
  public void actionPerformed(ActionEvent ae) {
    colorIndex++;
    colorIndex = colorIndex % smallRectColors.length;
    repaint();
  }
  
  
  private static void init(final File f, final int w, final int h, 
               final int font, final int wpm) throws FileNotFoundException {
    
    final Scanner scan = new Scanner(f);
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        createAndShowGUI(scan, w, h, font, wpm); 
      }
    });
  }
  
  
  private static void createAndShowGUI(Scanner scan, int width, int height,
                                                  int fontSize, int wpm) {
    
    JFrame f = new JFrame("Speed Reader");
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    
    System.out.println("width " + width + " heigth " + height);
    f.setSize(width, height);
        
    SpeedReader component = new SpeedReader(scan, fontSize, wpm);
    f.add(component);
    f.setVisible(true);
  }
  
  /**
   * @param args the command line arguments
   */
  public static void main(String[] args) throws IOException {
    
    //COMMAND LINE ARGS: implement them
    // provided code only handles speed, other values are hard-coded
    
    // Your code should take 5 command line arguments
    // args[0] filename
    // args[1] width of the panel
    // args[2] height of the panel
    // args[3] font size
    // args[4] speed at which the text should be displayed (word per min)
    
    System.out.println("Usage: run SpeedReader <filename> <width> <height>"
                         + "<font size> <wpm>");
    
    int wpm = 60;
    
    if  (args.length > 0) { 
      wpm = Integer.parseInt(args[0]);
    } 
    
    init(new File("test.a"), FRAME_WIDTH, FRAME_HEIGHT, 18, wpm);
  }
  
}

